﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using ComiRTEXDLL = ComiRTEX.SafeNativeMethods;
using ComiDef = ComiLib.NEMO_Def.SafeNativeMethods;

namespace HommingExample
{


    public partial class Homming : Form
    {
        private static int Axis; //선택한 축의 번호

        private static double Feedback_pos;
        private static double Command_pos;
        private static int Axis_isbusy;

        private static int MIO_PEL;
        private static int MIO_NEL;
        private static int MIO_ORG;

        private static int Homemode;
        private static int Homedir;
        private static double Offset;

        private static int PhaseID;
        private static int SpeedMode;
        private static double WorkSpeed;
        private static double Acceleration;
        private static double Deceleration;


        private static int ERRORCODE;
        private static int NumDevice = 0;
        private static int BoardID = 0;
        private static int nNumAxes = 0;

        public Homming()
        {
            InitializeComponent();
        }
        void InitMotionDevices()
        {
            // SERVO-ON //
            ComiDef.SlaveInfo SlaveInfo = new ComiDef.SlaveInfo();
            SlaveInfo.ModuleType = new ushort[9];

            for (int i = 0; i < nNumAxes; i++)
            {
                ERRORCODE = ComiRTEXDLL.GnSetServoOn(BoardID, i, 1);

                if(ERRORCODE != (int)ComiDef.ERROR_CODES.ERR_NONE)
                {
                    MessageBox.Show("GnSetServoOn() Error / Error code = " + ((ComiDef.ERROR_CODES)ERRORCODE).ToString());
                }
            }
        }

        void RefreshSpeedSetup()
        {
            ERRORCODE = ComiRTEXDLL.HomeGetSpeedPattern(BoardID, Axis, PhaseID, ref SpeedMode, ref WorkSpeed, ref Acceleration, ref Deceleration);

            if (ERRORCODE != (int)ComiDef.ERROR_CODES.ERR_NONE)
            {
                MessageBox.Show("HomeGetSpeedPattern() Error / Error code = " + ((ComiDef.ERROR_CODES)ERRORCODE).ToString());
            }

            comboBox_speedmode.SelectedIndex = SpeedMode;
            textBox_workspeed.Text = WorkSpeed.ToString();
            textBox_acceleration.Text = Acceleration.ToString();
            textBox_deceleration.Text = Deceleration.ToString();
        }

        void RefreshUI()
        {
            //Status Monitor
            ComiRTEXDLL.StGetPosition(BoardID, Axis, 0, ref Command_pos);
            label_command.Text = Command_pos.ToString();
            ComiRTEXDLL.StGetPosition(BoardID, Axis, 1, ref Feedback_pos);
            label_feedback.Text = Feedback_pos.ToString();

            ComiRTEXDLL.HomeIsBusy(BoardID, Axis, ref Axis_isbusy).ToString();
            if (Axis_isbusy == 0)
            {
                label_homeisbusy.Text = "Not Busy";
            }
            else
            {
                label_homeisbusy.Text = "Busy";
            }

            //MIO Settings
            ComiRTEXDLL.CfgGetMioProperty(BoardID, Axis, (int)ComiDef.MIOID.PEL_LOGIC, ref MIO_PEL);
            comboBox_PEL.SelectedIndex = MIO_PEL;
            ComiRTEXDLL.CfgGetMioProperty(BoardID, Axis, (int)ComiDef.MIOID.NEL_LOGIC, ref MIO_NEL);
            comboBox_NEL.SelectedIndex = MIO_NEL;
            ComiRTEXDLL.CfgGetMioProperty(BoardID, Axis, (int)ComiDef.MIOID.ORG_LOGIC, ref MIO_ORG);
            comboBox_ORG.SelectedIndex = MIO_ORG;


            //Home Settings
            ComiRTEXDLL.HomeGetConfig(BoardID, Axis, 0, ref Homemode);
            if (Homemode == 1)
            {
                comboBox_homemode.SelectedIndex = 0;
            }
            else if (Homemode == 3)
            {
                comboBox_homemode.SelectedIndex = 1;
            }

            ComiRTEXDLL.HomeGetConfig(BoardID, Axis, 1, ref Homedir);
            comboBox_homedir.SelectedIndex = Homedir;

            ComiRTEXDLL.HomeGetOffset(BoardID, Axis, ref Offset);
            textBox_offset.Text = Offset.ToString();


            //Speed Setup
            PhaseID = 0;
            comboBox_phaseid.SelectedIndex = PhaseID;
            RefreshSpeedSetup();

        }

       

        private void Homming_Load(object sender, EventArgs e)
        {
            ERRORCODE = (int)ComiDef.ERROR_CODES.ERR_NONE;
            // Load Motion & DIO devices //
            

            ERRORCODE = ComiRTEXDLL.GnLoadDevice(ref NumDevice, ref BoardID, ref nNumAxes);
            
            if (ERRORCODE != (int)ComiDef.ERROR_CODES.ERR_NONE)
            {
                MessageBox.Show("GnLoadDevice() Error / Error code = " + ((ComiDef.ERROR_CODES)ERRORCODE).ToString());
            }

            InitMotionDevices();

            //Axis 설정
            comboBox_axis.Items.Clear();
            for(int i = 0; i < nNumAxes; i++)
            {
                comboBox_axis.Items.Add(i);
            }
            if (nNumAxes != 0)
            {
                comboBox_axis.SelectedIndex = 0;
                Axis = comboBox_axis.SelectedIndex;
            }
            else
            {
                MessageBox.Show("설정된 축이 없습니다.");
            }
            

            RefreshUI();

            timer1.Start();
        }

        private void Timer1_Tick(object sender, EventArgs e)
        {
            ComiRTEXDLL.StGetPosition(BoardID, Axis, (int)ComiDef.CmCntr.COMM, ref Command_pos);
            label_command.Text = Command_pos.ToString();
            ComiRTEXDLL.StGetPosition(BoardID, Axis, (int)ComiDef.CmCntr.FEED, ref Feedback_pos);
            label_feedback.Text = Feedback_pos.ToString();
            
            ComiRTEXDLL.HomeIsBusy(BoardID, Axis, ref Axis_isbusy).ToString();
            if(Axis_isbusy == 0)
            {
                label_homeisbusy.Text = "Not Busy";
            }
            else
            {
                label_homeisbusy.Text = "Busy";
            }
        }

        //인덱스 변화할때마다 변수 업데이트해주는거 넣기
        private void Axis_Change(object sender, EventArgs e)
        {
            Axis = comboBox_axis.SelectedIndex;
            RefreshUI();
        }

        private void Homemode_Change(object sender, EventArgs e)
        {
            if (comboBox_homemode.SelectedIndex == 0)
            {
                Homemode = 1;
            }
            else
            {
                Homemode = 3;
            }
            
        }

        private void HomeDirection_Change(object sender, EventArgs e)
        {
            Homedir = comboBox_homedir.SelectedIndex;
        }

        private void PhaseID_Change(object sender, EventArgs e)
        {
            PhaseID = comboBox_phaseid.SelectedIndex;
            RefreshSpeedSetup();
        }

        private void SpeedMode_Change(object sender, EventArgs e)
        {
            SpeedMode = comboBox_speedmode.SelectedIndex;
        }
        
        //버튼 핸들링 넣기
        private void button_resetposition_Click(object sender, EventArgs e)
        {
            Command_pos = 0;
            ComiRTEXDLL.StSetPosition(BoardID, Axis, (int)ComiDef.CmCntr.COMM, Command_pos);
            label_command.Text = Command_pos.ToString();

            Feedback_pos = 0;
            ComiRTEXDLL.StSetPosition(BoardID, Axis, (int)ComiDef.CmCntr.FEED, Feedback_pos);
            label_feedback.Text = Feedback_pos.ToString();
        }

        private void button_applyhomesetting_Click(object sender, EventArgs e)
        {
            ComiRTEXDLL.HomeSetConfig(BoardID, Axis, 0, Homemode);
            ComiRTEXDLL.HomeSetConfig(BoardID, Axis, 1, Homedir);
            
            Offset = double.Parse(textBox_offset.Text);
            ComiRTEXDLL.HomeSetOffset(BoardID, Axis, Offset);
        }

        private void button_applyspeedsetup_Click(object sender, EventArgs e)
        {
            WorkSpeed = double.Parse(textBox_workspeed.Text);
            Acceleration = double.Parse(textBox_acceleration.Text);
            Deceleration = double.Parse(textBox_deceleration.Text);
            ComiRTEXDLL.HomeSetSpeedPattern(BoardID, Axis, PhaseID, SpeedMode, WorkSpeed, Acceleration, Deceleration);
        }

        private void button_homemove_Click(object sender, EventArgs e)
        {
            ERRORCODE = ComiRTEXDLL.HomeMove(BoardID, Axis, 0);

            if (ERRORCODE != (int)ComiDef.ERROR_CODES.ERR_NONE)
            {
                MessageBox.Show("HomeMove() Error / Error code = " + ((ComiDef.ERROR_CODES)ERRORCODE).ToString());
            }
        }

        private void button_stop_Click(object sender, EventArgs e)
        {
            ERRORCODE = ComiRTEXDLL.SxStop(BoardID, Axis);

            if (ERRORCODE != (int)ComiDef.ERROR_CODES.ERR_NONE)
            {
                MessageBox.Show("SxStop() Error / Error code = " + ((ComiDef.ERROR_CODES)ERRORCODE).ToString());
            }
        }

        private void PEL_CHANGE(object sender, EventArgs e)
        {
            MIO_PEL = comboBox_PEL.SelectedIndex;
        }

        private void NEL_CHANGE(object sender, EventArgs e)
        {
            MIO_NEL = comboBox_NEL.SelectedIndex;
        }

        private void ORG_CHANGE(object sender, EventArgs e)
        {
            MIO_ORG = comboBox_ORG.SelectedIndex;
        }

        private void button_applymio_Click(object sender, EventArgs e)
        {
            ComiRTEXDLL.CfgSetMioProperty(BoardID, Axis, (int)ComiDef.MIOID.PEL_LOGIC, MIO_PEL);
            ComiRTEXDLL.CfgSetMioProperty(BoardID, Axis, (int)ComiDef.MIOID.NEL_LOGIC, MIO_NEL);
            ComiRTEXDLL.CfgSetMioProperty(BoardID, Axis, (int)ComiDef.MIOID.ORG_LOGIC, MIO_ORG);
        }
    }
}
